%module proj

%{
#include "projects.h"
%}

%include "typemaps.i"

#ifdef SWIGGUILE
%include "list-vector.i"

%scheme %{
;; SWIG does not emit define-module for Module linkage
(define-module (proj))
(load-extension "libguile_proj.so" "scm_init_proj_module")

(define-public (degrees-to-radians d) (* 3.14159265358979323846 (/ d 180.0)))
(define-public (radians-to-degrees r) (* 180.0 (/ r 3.14159265358979323846)))

(define *pj-utm84* (pj-init '("proj=utm" "lon_0=69W")))

;; geodetic in degrees, positive east longitude
;; utm in meters

(define-public (geodetic->utm19-84 lat lon)
  (let*
      ((g (new-projUV (degrees-to-radians lon)
		      (degrees-to-radians lat)))
       (u (pj-fwd g *pj-utm84*)))
    (list
     (projUV-u-get u)
     (projUV-v-get u))))

(define-public (utm19-84->geodetic e n)
  (let*
      ((u (new-projUV e n))
       (g (pj-inv u *pj-utm84*)))
    (list
     (radians-to-degrees (projUV-u-get g))
     (radians-to-degrees (projUV-v-get g)))))
%}
#endif

typedef void *projPJ;

projPJ pj_init(int LISTLENINPUT, char **LISTINPUT);
void pj_free(projPJ);

typedef struct { double u, v; } projUV;
%extend projUV {
	projUV(double u, double v) {
		projUV *puv;
		puv = (projUV *) malloc(sizeof(puv));
		puv->u = u;
		puv->v = v;
		return puv;
	}
	~projUV() {
		free(self);
	}
};

projUV pj_fwd(projUV, projPJ);
projUV pj_inv(projUV, projPJ);
